@@ -19,10 +19,10 @@ def getAtomsToMove(i,j,j_,ToBeMove,neighbors,ring=False):
     return ToBeMove,ring
 
 
-def get_group(i,j,r,atoms):
+def get_group(i,j,atoms):
     positions = atoms.get_positions()
     center    = 0.5*(positions[i] + positions[j]) 
-    vij = r[i][j]
+    vij = positions[j] - positions[i]
     rij = np.sqrt(np.sum(np.square(vij)))
     vij = vij/rij
 
@@ -31,6 +31,7 @@ def get_group(i,j,r,atoms):
     for i_,x in enumerate(positions):
         v    = x - center
         dot_ = np.dot(v,vij)
+        # print(v,vij,dot_)
         if dot_<=0.0:
            group_i.append(i_)
         else:
@@ -220,7 +221,7 @@ def check_zmat(atoms=None,rmin=0.8,rmax=1.2,angmax=15.0,
 
 
 class AtomDance(object):
-  def __init__(self,atoms=None,poscar=None,nn=True,
+  def __init__(self,atoms=None,poscar=None,nn=True,ffield='ffield.json',
                rotAng=40.0,angmax=30.0,freeatoms=None,FirstAtom=None,
                rmin=0.4,rmax=1.25,botol=0.0):
       self.rmin          = rmin
@@ -237,7 +238,7 @@ class AtomDance(object):
             atoms  = read(poscar)
 
       self.ir = IRFF_NP(atoms=atoms,
-                        libfile='ffield.json',
+                        libfile=ffield,
                         rcut=None,
                         nn=nn)
       self.natom     = self.ir.natom
@@ -259,7 +260,8 @@ class AtomDance(object):
              label_dic[sp]  = 1
       self.label = ''
       for sp in spec:
-          self.label += sp+str(label_dic[sp])
+          if sp in label_dic:
+             self.label += sp+str(label_dic[sp])
 
       self.ir.calculate_Delta(atoms)
       self.InitBonds = getBonds(self.natom,self.ir.r,self.rmax*self.ir.re,self.ir.bo0,
@@ -300,46 +302,48 @@ class AtomDance(object):
       self.zmat_id    = []
       specs           = atoms.get_chemical_symbols()
       self.mols       = molecules(self.natom,specs,atoms.positions,
-                                 cell=atoms.cell,
-                                 table=self.neighbors)
+                                  cell=atoms.cell,
+                                  table=self.neighbors)
       for m in self.mols:
           if len(m.mol_index)==1:
-             continue
-          for i in m.mol_index:
-              first = i
-              if len(self.neighbors[i])==1:
+             self.zmat_index.append([-1,-1,-1])
+             self.zmat_id.append(m.mol_index[0])
+          else:
+             for i in m.mol_index:
                  first = i
-                 break 
-          if not self.FirstAtom is None:
-             if self.FirstAtom in m.mol_index:
-                first = self.FirstAtom
-          self.zmat_index.append([-1,-1,-1])
-          self.zmat_id.append(first)
+                 if len(self.neighbors[i])==1:
+                    first = i
+                    break 
+             if not self.FirstAtom is None:
+                if self.FirstAtom in m.mol_index:
+                   first = self.FirstAtom
+             self.zmat_index.append([-1,-1,-1])
+             self.zmat_id.append(first)
 
-          second = None
-          if m.natom>=2:
-             second= self.neighbors[first][0]
-             self.zmat_index.append([first,-1,-1])
-             self.zmat_id.append(second)
+             second = None
+             if m.natom>=2:
+                second= self.neighbors[first][0]
+                self.zmat_index.append([first,-1,-1])
+                self.zmat_id.append(second)
 
-          third = None
-          if m.natom>=3:
-             for i in self.neighbors[second]:
-                 if i != first:
-                    if not third is None:
-                       if len(self.neighbors[i])>len(self.neighbors[third]):
+             third = None
+             if m.natom>=3:
+                for i in self.neighbors[second]:
+                    if i != first:
+                       if not third is None:
+                          if len(self.neighbors[i])>len(self.neighbors[third]):
+                             third = i
+                       else:      
                           third = i
-                    else:      
-                       third = i
 
-          if not third is None:
-             self.zmat_index.append([second,first,-1])  
-             self.zmat_id.append(third)
-
-             self.zmat_id,self.zmat_index = add_zmat_atom(second,first,third,
-                                  self.neighbors,self.zmat_id,self.zmat_index)
-             self.zmat_id,self.zmat_index = add_zmat_atom(third,second,first,
-                                  self.neighbors,self.zmat_id,self.zmat_index)
+             if not third is None:
+                self.zmat_index.append([second,first,-1])  
+                self.zmat_id.append(third)
+ 
+                self.zmat_id,self.zmat_index = add_zmat_atom(second,first,third,
+                                     self.neighbors,self.zmat_id,self.zmat_index)
+                self.zmat_id,self.zmat_index = add_zmat_atom(third,second,first,
+                                     self.neighbors,self.zmat_id,self.zmat_index)
 
   def zmat_to_cartation(self,atoms,zmat):
       for i in range(len(self.zmat_id)):
@@ -426,7 +430,7 @@ class AtomDance(object):
                 group_ = groups[j]
                 axis   = [i,j]
                 o      = j
-          images = self.a.rotate(atms=group_,axis=axis,o=o,rang=self.rotAng,nbin=30,traj='md.traj')
+          images = self.rotate(atms=group_,axis=axis,o=o,rang=self.rotAng,nbin=30,traj='md.traj')
 
   def get_groups(self):
       groups = [None for i in range(self.natom)]
@@ -526,7 +530,8 @@ class AtomDance(object):
                           v[i] = v[i] + 0.05*vij
                        else:
                           v[i] = v[i] - 0.05*vij
-      atoms.set_velocities(v)
+      if not v is None:
+         atoms.set_velocities(v)
       return atoms
 
 
@@ -544,7 +549,7 @@ class AtomDance(object):
       ig      = len(group_i)
 
       if ring:
-         group_i,group_j = get_group(i,j,self.ir.r,atoms)
+         group_i,group_j = get_group(i,j,atoms)
 
       vij   = self.ir.vr[j][i]/self.ir.r[i][j]
       massi = 0.0
@@ -576,8 +581,7 @@ class AtomDance(object):
 
   def zmat_relax(self,atoms,nbin=10,relax_step=None,
                  zmat_variable=None,zvlo=None,zvhi=None,
-                 traj='zmat.traj'):
-      relaxlog = ''
+                 traj='zmat.traj',relaxlog = ''):
       zmatrix  = np.array(self.get_zmatrix(atoms))
       initz    = np.array(self.InitZmat)         # target
       relax_step_ = nbin if relax_step is None else relax_step
@@ -974,7 +978,8 @@ class AtomDance(object):
       return images
 
 
-  def swing_group(self,ang=None,group=None,rang=20,nbin=30,scale=1.2,traj=None):
+  def swing_group(self,ang=None,group=None,rang=20.0,nbin=30,scale=1.2,
+                  neighbors=None,traj=None):
       i,j,k = ang
       atoms = self.ir.atoms
       self.ir.calculate_Delta(atoms)
@@ -990,13 +995,27 @@ class AtomDance(object):
       if rk<0.0000001:
          uk = np.array([1.0,0.0,0.0])
       else:
-         uk  = uk/rk       
-      images = self.rotate(atms=group,axis_vector=uk,o=j,rang=rang,nbin=nbin,
-                            traj=traj,scale=scale)
+         uk  = uk/rk   
+
+      if group is None:
+         if neighbors is None:
+            # self.ir.calculate_Delta(atoms)
+            neighbors = getNeighbor(self.natom,self.ir.r,scale*self.ir.re,self.ir.bo0)
+         if group is None:
+            group      = []
+            group,ring = getAtomsToMove(j,k,k,group,neighbors)
+         else:
+            ring       = False
+         if ring:
+            group = [k]
+
+      images = self.rotate(atms=group,axis_vector=uk,o=j,rang=rang,
+                           nbin=nbin,traj=traj,scale=scale)
       return images
 
 
-  def rotate(self,atms=None,axis=None,axis_vector=None,o=None,rang=20.0,nbin=10,traj=None,scale=1.2):
+  def rotate(self,atms=None,axis=None,axis_vector=None,o=None,rang=20.0,
+             nbin=10,traj=None,scale=1.2):
       da = 2.0*rang/nbin
       atoms = self.ir.atoms
       self.ir.calculate_Delta(atoms)
@@ -1012,7 +1031,7 @@ class AtomDance(object):
       else:
          uk    = axis_vector
 
-      a_     = -rang
+      a_       =  -rang
       for i in range(nbin+1):
           atoms_ = atoms.copy()
           for atomk in atms:
@@ -1086,7 +1105,7 @@ class AtomDance(object):
       return images
 
 
-  def continous(self,atoms,dr=0.001,da=1.0):
+  def continous(self,atoms,dr=0.001,da=1.0,nbin=10,traj='md.traj'):
       ''' check the continous of the PES of the current configuration '''
       de = np.zeros((self.natom,3,2))
       # images = []
@@ -1119,7 +1138,26 @@ class AtomDance(object):
                      # images.append(atoms_)
                      del atoms_
                      del zmat_
-      return de # ,images
+      zi = np.argmax(np.abs(de[:,0,0]))
+      zj = np.argmax(np.abs(de[:,0,1]))
+      if abs(de[zi,0,0]) >abs(de[zj,0,1]):
+         zv = (zi,0,)
+         l  = 0
+      else:
+         zv = (zj,0)
+         l  = 1
+      atomi = self.zmat_id[i]
+      atomj = self.zmat_index[i][0]
+      zvlo  = - 0.05*zmatrix[zv]
+      zvhi  =   0.05*zmatrix[zv]
+      # print(zvlo,zvhi)
+      relaxlog  = 'MD step is littile than 3, checking the continous of the zmatmix,\n' 
+      relaxlog += 'the dE/dr between atoms {:3d} {:3d} is {:6.4f}.\n'.format(atomi,atomj,
+                                                                  float(de[zv[0]][zv[1]][l]))
+      images,relaxlog = self.zmat_relax(atoms=atoms,zmat_variable=zv,nbin=nbin,
+                                        zvlo=zvlo,zvhi=zvhi,traj=traj,
+                                        relaxlog=relaxlog)
+      return images,relaxlog
 
 
   def scan_pes(self,i,n,neighbor,atoms,images,r,nbin=[5],dr=[0.1]):
