@@ -85,7 +85,7 @@ class IRFF(Calculator):
                vdwcut=10.0,
                atol=0.001,
                hbtol=0.001,
-               nn=False,vdwnn=False,
+               nn=False,# vdwnn=False,
                messages=1,
                hbshort=6.75,hblong=7.5,
                autograd=True,
@@ -98,7 +98,7 @@ class IRFF(Calculator):
       self.natom        = len(self.atom_name)
       self.spec         = []
       self.nn           = nn
-      self.vdwnn        = vdwnn
+      # self.vdwnn      = vdwnn
       self.EnergyFunction = 0
       self.autograd     = autograd
       self.messages     = messages 
@@ -119,6 +119,10 @@ class IRFF(Calculator):
          self.mf_layer       = j['mf_layer']
          self.be_layer       = j['be_layer']
          self.vdw_layer      = j['vdw_layer']
+         if not self.vdw_layer is None:
+            self.vdwnn       = True
+         else:
+            self.vdwnn       = False
          rcut                = j['rcut']
          rcuta               = j['rcutBond']
          lf.close()
@@ -140,6 +144,7 @@ class IRFF(Calculator):
          self.emol     = 0.0
          rcut          = None
          rcuta         = None
+         self.vdwnn    = False
 
       if m is None:
          self.nn=False
@@ -372,7 +377,7 @@ class IRFF(Calculator):
 
 
           if self.MessageFunction==1:
-             F   = 2.0*Fi*Fj
+             F   = Fi*Fj
              self.Hsi.append(self.Hsi[t-1]*F)
              self.Hpi.append(self.Hpi[t-1]*F)
              self.Hpp.append(self.Hpp[t-1]*F)
@@ -384,11 +389,8 @@ class IRFF(Calculator):
              self.Hsi.append(self.Hsi[t-1]*Fsi)
              self.Hpi.append(self.Hpi[t-1]*Fpi)
              self.Hpp.append(self.Hpp[t-1]*Fpp)
-          elif self.MessageFunction==3:
-             F   = Fi*Fj
-             self.Hsi.append(self.Hsi[t-1]*F)
-             self.Hpi.append(self.Hpi[t-1]*F)
-             self.Hpp.append(self.Hpp[t-1]*F)
+          else:
+             raise NotImplementedError('-  Message function not supported yet!')
           self.H.append(self.Hsi[t]+self.Hpi[t]+self.Hpp[t])
           self.D.append(torch.sum(self.H[t],1) )
 
@@ -410,16 +412,15 @@ class IRFF(Calculator):
       Dbi        = Di_ - self.bo0
       Dbj        = Dj_ - self.bo0
 
-      if self.EnergyFunction==3: 
+      if self.EnergyFunction==1: 
          e_ = self.f_nn('fe',[self.bosi,self.bopi,self.bopp],layer=self.be_layer[1])
          self.esi = self.bo0*e_
-      elif self.EnergyFunction==1:
-         self.esi = self.f_nn('fe',[self.bosi,self.bopi,self.bopp],layer=self.be_layer[1])
       elif self.EnergyFunction==2:
-         Fi = self.f_nn('fe',[Dbj,Dbi,self.bosi],layer=self.be_layer[1])
+         Fi = self.f_nn('fe',[Dbj,Dbi,self.bo0],layer=self.be_layer[1])
          Fj = torch.transpose(Fi,0,1)
-         F  = Fi*Fj
-         self.esi   = self.bosi*F
+         self.esi  = Fi*Fj*self.bo0
+      elif self.EnergyFunction==3:
+         self.esi = self.f_nn('fe',[self.bosi,self.bopi,self.bopp],layer=self.be_layer[1])
 
 
   def get_ebond(self,cell,rcell,positions):
@@ -447,8 +448,13 @@ class IRFF(Calculator):
       self.fbo   = taper(self.bo0,rmin=self.atol,rmax=2.0*self.atol) 
       self.fhb   = taper(self.bo0,rmin=self.hbtol,rmax=2.0*self.hbtol) 
 
-      if self.EnergyFunction==3 or self.EnergyFunction==1:
+      if self.EnergyFunction==1 or self.EnergyFunction==2:
          self.ebond = - self.P['Desi']*self.esi
+      elif self.EnergyFunction==3:
+         self.sieng =   self.P['Desi']*self.esi*self.bosi
+         self.pieng =   self.P['Depi']*self.esi*self.bopi
+         self.ppeng =   self.P['Depp']*self.esi*self.bopp
+         self.ebond = - self.sieng - self.pieng - self.ppeng
       else:
           if self.nn:
              self.sieng = torch.mul(self.P['Desi'],self.esi)
@@ -526,8 +532,11 @@ class IRFF(Calculator):
       sbo   = self.Dpi[self.angj]
       pbo   = self.PBO[self.angj]
       rnlp  = self.nlp[self.angj]
-
-      SBO   = sbo - (1.0-pbo)*(dang+self.P['val8']*rnlp)    
+      
+      # if self.nn:
+      #    SBO= sbo   
+      # else:
+      SBO= sbo - (1.0-pbo)*(dang+self.P['val8']*rnlp)  
       
       ok    = torch.logical_and(SBO<=1.0,SBO>0.0)
       S1    = torch.where(ok,SBO,torch.full_like(SBO,0.0))         #  0< sbo < 1                  
@@ -541,8 +550,11 @@ class IRFF(Calculator):
       SBO12 = torch.where(ok,2.0-torch.pow(S2,self.P['val9']),torch.full_like(S2,0.0)) #  1< sbo <2
       SBO2  = torch.where(SBO>2.0,torch.full_like(S2,1.0),torch.full_like(S2,0.0))                         #     sbo >2
 
-      self.SBO3   = SBO01+SBO12+2.0*SBO2
-      thet_ = 180.0 - torch.mul(self.P['theta0'],(1.0-torch.exp(-self.P['val10']*(2.0-self.SBO3))))
+      self.SBO3  = SBO01+SBO12+2.0*SBO2
+      # if self.nn:
+      #    thet_ = torch.mul(self.P['theta0'],(1.0-torch.exp(-self.P['val10']*(2.0-self.SBO3))))
+      # else:
+      thet_      = 180.0 - torch.mul(self.P['theta0'],(1.0-torch.exp(-self.P['val10']*(2.0-self.SBO3))))
       self.thet0 = thet_/57.29577951
 
 
@@ -738,14 +750,15 @@ class IRFF(Calculator):
 
 
   def get_tap(self,r):
+      tpc = 1.0+torch.div(-35.0,self.vdwcut**4.0)*torch.pow(r,4.0)+ \
+            torch.div(84.0,self.vdwcut**5.0)*torch.pow(r,5.0)+ \
+            torch.div(-70.0,self.vdwcut**6.0)*torch.pow(r,6.0)+ \
+            torch.div(20.0,self.vdwcut**7.0)*torch.pow(r,7.0)
       if self.vdwnn:
-         tp = self.f_nn('fv',[r],layer=self.vdw_layer[1])
+         tp = self.f_nn('fv',[r],layer=self.vdw_layer[1]) # /self.P['rvdw']
       else:
-         tp = 1.0+torch.div(-35.0,self.vdwcut**4.0)*torch.pow(r,4.0)+ \
-              torch.div(84.0,self.vdwcut**5.0)*torch.pow(r,5.0)+ \
-              torch.div(-70.0,self.vdwcut**6.0)*torch.pow(r,6.0)+ \
-              torch.div(20.0,self.vdwcut**7.0)*torch.pow(r,7.0)
-      return  tp
+         tp = tpc
+      return tp,tpc
 
 
   def get_evdw(self,cell_tensor):
@@ -769,14 +782,14 @@ class IRFF(Calculator):
                      fv = fv_*self.d2
 
                   f_13 = self.f13(r)
-                  tpv  = self.get_tap(r)
+                  tpv,tpc  = self.get_tap(r)
 
                   expvdw1 = torch.exp(0.5*self.P['alfa']*(1.0-torch.div(f_13,2.0*self.P['rvdw'])))
                   expvdw2 = torch.square(expvdw1) 
                   self.evdw  += fv*tpv*self.P['Devdw']*(expvdw2-2.0*expvdw1)
 
                   rth         = torch.pow(r3+gm3,1.0/3.0)                                      # ecoul
-                  self.ecoul += torch.div(fv*tpv*self.qij,rth)
+                  self.ecoul += torch.div(fv*tpc*self.qij,rth)
                   nc += 1
 
       self.Evdw  = torch.sum(self.evdw)
@@ -1124,7 +1137,10 @@ class IRFF(Calculator):
       # if self.EnergyFunction==1:
       #    pres = ['fesi','fepi','fepp','fsi','fpi','fpp','fv']
       # else:
-      pres = ['fe','fsi','fpi','fpp','fv']
+      pres = ['fe','fsi','fpi','fpp']
+      if self.vdwnn:
+         pres.append('fv')
+
       for t in range(1,self.messages+1):
           pres.append('f'+str(t))
 
