@@ -77,7 +77,7 @@ class IRFF_NP(object):
   def __init__(self,atoms=None,
                libfile='ffield',
                vdwcut=10.0,
-               nn=False,vdwnn=False,
+               nn=False,# vdwnn=False,
                messages=1,
                hbshort=6.75,hblong=7.5,
                label="IRFF", **kwargs):
@@ -88,7 +88,7 @@ class IRFF_NP(object):
       self.natom        = len(self.atom_name)
       self.spec         = []
       self.nn           = nn
-      self.vdwnn        = vdwnn
+      # self.vdwnn      = vdwnn
       self.EnergyFunction = 0
       self.messages     = messages
       self.safety_value = 0.000000001
@@ -106,12 +106,16 @@ class IRFF_NP(object):
          self.messages       = j['messages']
          self.EnergyFunction = j['EnergyFunction']
          self.MessageFunction= j['MessageFunction']
-         self.bo_layer = j['bo_layer']
-         self.mf_layer = j['mf_layer']
-         self.be_layer = j['be_layer']
-         self.vdw_layer= j['vdw_layer']
-         rcut          = j['rcut']
-         rcuta         = j['rcutBond']
+         self.bo_layer       = j['bo_layer']
+         self.mf_layer       = j['mf_layer']
+         self.be_layer       = j['be_layer']
+         self.vdw_layer      = j['vdw_layer']
+         if not self.vdw_layer is None:
+            self.vdwnn       = True
+         else:
+            self.vdwnn       = False
+         rcut                = j['rcut']
+         rcuta               = j['rcutBond']
          lf.close()
          self.init_bonds()
       else:
@@ -121,6 +125,7 @@ class IRFF_NP(object):
          self.bo_layer    = None
          rcut             = None
          rcuta            = None
+         self.vdwnn       = False
 
       if m is None:
          self.nn=False
@@ -367,7 +372,7 @@ class IRFF_NP(object):
 
           if self.MessageFunction==1:
              Fj    = np.transpose(Fi,[1,0])
-             F     = 2.0*Fi*Fj
+             F     = Fi*Fj
              self.F.append(F)
              self.Hsi.append(self.Hsi[t-1]*F)
              self.Hpi.append(self.Hpi[t-1]*F)
@@ -382,13 +387,8 @@ class IRFF_NP(object):
              self.Hsi.append(self.Hsi[t-1]*Fsi)
              self.Hpi.append(self.Hpi[t-1]*Fpi)
              self.Hpp.append(self.Hpp[t-1]*Fpp)
-          elif self.MessageFunction==3:
-             Fj    = np.transpose(Fi,[1,0])
-             F     = Fi*Fj
-             self.F.append(F)
-             self.Hsi.append(self.Hsi[t-1]*F)
-             self.Hpi.append(self.Hpi[t-1]*F)
-             self.Hpp.append(self.Hpp[t-1]*F)
+          else:
+             raise NotImplementedError('-  Message function not supported yet!')
           self.H.append(self.Hsi[t]+self.Hpi[t]+self.Hpp[t])
           self.D.append(np.sum(self.H[t],axis=1))  
 
@@ -410,16 +410,15 @@ class IRFF_NP(object):
       Dbi        = Di_ - self.bo0
       Dbj        = Dj_ - self.bo0
 
-      if self.EnergyFunction==3: 
+      if self.EnergyFunction==1: 
          e_ = self.f_nn('fe',[self.bosi,self.bopi,self.bopp],layer=self.be_layer[1])
          self.esi = self.bo0*e_
-      elif self.EnergyFunction==1:
-         self.esi = self.f_nn('fe',[self.bosi,self.bopi,self.bopp],layer=self.be_layer[1])
       elif self.EnergyFunction==2:
-         Fi = self.f_nn('fe',[Dbj,Dbi,self.bosi],layer=self.be_layer[1])
+         Fi = self.f_nn('fe',[Dbj,Dbi,self.bo0],layer=self.be_layer[1])
          Fj = np.transpose(Fi,[1,0])
-         F  = Fi*Fj
-         self.esi   = self.bosi*F
+         self.esi = Fi*Fj*self.bo0
+      elif self.EnergyFunction==3:
+         self.esi = self.f_nn('fe',[self.bosi,self.bopi,self.bopp],layer=self.be_layer[1])
 
 
   def get_ebond(self,cell,rcell,positions):
@@ -447,8 +446,13 @@ class IRFF_NP(object):
       self.fbo   = taper(self.bo0,rmin=self.atol,rmax=2.0*self.atol) 
       self.fhb   = taper(self.bo0,rmin=self.hbtol,rmax=2.0*self.hbtol) 
 
-      if self.EnergyFunction==3 or self.EnergyFunction==1:
+      if self.EnergyFunction==1 or self.EnergyFunction==2:
          self.ebond = - self.P['Desi']*self.esi
+      elif self.EnergyFunction==3:
+         self.sieng =   self.P['Desi']*self.esi*self.bosi
+         self.pieng =   self.P['Depi']*self.esi*self.bopi
+         self.ppeng =   self.P['Depp']*self.esi*self.bopp
+         self.ebond = - self.sieng - self.pieng - self.ppeng
       else:
          if self.nn:
             self.sieng = np.multiply(self.P['Desi'],self.esi)
@@ -520,12 +524,18 @@ class IRFF_NP(object):
 
 
   def get_theta0(self,dang):
-      sbo   = np.squeeze(self.Dpi[self.angj])
-      pbo   = np.squeeze(self.PBO[self.angj])
-      rnlp  = np.squeeze(self.nlp[self.angj])
+      self.dang  = dang
+      self.sbo   = np.squeeze(self.Dpi[self.angj])
+      self.pbo   = np.squeeze(self.PBO[self.angj])
+      rnlp       = np.squeeze(self.nlp[self.angj])
+
+      # if self.nn:
+      #    SBO   = self.sbo 
+      #    self.SBO = SBO
+      # else:
+      SBO   = self.sbo - (1.0-self.pbo)*(dang+self.P['val8']*rnlp)  
+      self.SBO = SBO
 
-      SBO   = sbo - (1.0-pbo)*(dang+self.P['val8']*rnlp)    
-      
       ok    = np.logical_and(SBO<=1.0,SBO>0.0)
       S1    = np.where(ok,SBO,0.0)                             #  0< sbo < 1                  
       SBO01 = np.where(ok,np.power(S1,self.P['val9']),0.0)
@@ -537,8 +547,12 @@ class IRFF_NP(object):
       SBO12 = np.where(ok,2.0-np.power(S2,self.P['val9']),0.0)  #  1< sbo <2
       SBO2  = np.where(SBO>2.0,1.0,0.0)                         #     sbo >2
 
-      self.SBO3   = SBO01+SBO12+2.0*SBO2
-      thet_ = 180.0 - self.P['theta0']*(1.0-np.exp(-self.P['val10']*(2.0-self.SBO3)))
+      self.SBO3  = SBO01+SBO12+2.0*SBO2
+      theta_     = self.P['theta0']*(1.0-np.exp(-self.P['val10']*(2.0-self.SBO3)))
+      #if self.nn:
+      #   thet_  = self.P['theta0']*(1.0-np.exp(-self.P['val10']*(2.0-self.SBO3)))
+      #else:
+      thet_      = 180.0 - self.P['theta0']*(1.0-np.exp(-self.P['val10']*(2.0-self.SBO3)))
       self.thet0 = thet_/57.29577951
 
 
@@ -738,14 +752,15 @@ class IRFF_NP(object):
 
 
   def get_tap(self,r):
+      tpc = 1.0+np.divide(-35.0,np.power(self.vdwcut,4.0))*np.power(r,4.0)+ \
+            np.divide(84.0,np.power(self.vdwcut,5.0))*np.power(r,5.0)+ \
+            np.divide(-70.0,np.power(self.vdwcut,6.0))*np.power(r,6.0)+ \
+            np.divide(20.0,np.power(self.vdwcut,7.0))*np.power(r,7.0)
       if self.vdwnn:
-         tp = self.f_nn('fv',[r],layer=self.vdw_layer[1])
+         tp = self.f_nn('fv',[r],layer=self.vdw_layer[1]) # /self.P['rvdw']
       else:
-         tp = 1.0+np.divide(-35.0,np.power(self.vdwcut,4.0))*np.power(r,4.0)+ \
-              np.divide(84.0,np.power(self.vdwcut,5.0))*np.power(r,5.0)+ \
-              np.divide(-70.0,np.power(self.vdwcut,6.0))*np.power(r,6.0)+ \
-              np.divide(20.0,np.power(self.vdwcut,7.0))*np.power(r,7.0)
-      return  tp
+         tp = tpc
+      return tp,tpc
 
 
   def get_evdw(self,cell):
@@ -769,14 +784,14 @@ class IRFF_NP(object):
                      fv = fv_*self.d2
 
                   f_13 = self.f13(r)
-                  tpv  = self.get_tap(r)
+                  tpv,tpc  = self.get_tap(r)
 
                   expvdw1 = np.exp(0.5*self.P['alfa']*(1.0-np.divide(f_13,2.0*self.P['rvdw'])))
                   expvdw2 = np.square(expvdw1) 
                   self.evdw  += fv*tpv*self.P['Devdw']*(expvdw2-2.0*expvdw1)
 
                   rth         = np.power(r3+gm3,1.0/3.0)                                      # ecoul
-                  self.ecoul += np.divide(fv*tpv*self.qij,rth)
+                  self.ecoul += np.divide(fv*tpc*self.qij,rth)
                   nc += 1
 
       self.Evdw  = np.sum(self.evdw)
@@ -784,13 +799,13 @@ class IRFF_NP(object):
 
   
   def get_ehb(self,cell):
-      self.BOhb   = np.squeeze(self.bo0[self.hbi,self.hbj])
-      fhb         = np.squeeze(self.fhb[self.hbi,self.hbj])
+      self.BOhb   = self.bo0[self.hbi,self.hbj] # np.squeeze(self.bo0[self.hbi,self.hbj])
+      fhb         = self.fhb[self.hbi,self.hbj] # np.squeeze(self.fhb[self.hbi,self.hbj])
 
-      rij         = np.squeeze(self.r[self.hbi,self.hbj])
-      rij2        = np.square(rij)
-      vrij        = np.squeeze(self.vr[self.hbi,self.hbj]) 
-      vrjk_       = np.squeeze(self.vr[self.hbj,self.hbk]) 
+      rij         = self.r[self.hbi,self.hbj]   # np.squeeze(self.r[self.hbi,self.hbj])
+      rij2        = rij # np.square(rij)
+      vrij        = self.vr[self.hbi,self.hbj]  # np.squeeze(self.vr[self.hbi,self.hbj]) 
+      vrjk_       = self.vr[self.hbj,self.hbk]  # np.squeeze(self.vr[self.hbj,self.hbk]) 
       self.Ehb    = 0.0
       
       for i in range(-1,2):
@@ -798,6 +813,7 @@ class IRFF_NP(object):
               for k in range(-1,2):
                   cell_  = cell[0]*i + cell[1]*j+cell[2]*k
                   vrjk   = vrjk_ + cell_
+
                   rjk2   = np.sum(np.square(vrjk),axis=1)
                   rjk    = np.sqrt(rjk2)
                   
@@ -1136,7 +1152,10 @@ class IRFF_NP(object):
       # if self.EnergyFunction==1:
       #    pres = ['fesi','fepi','fepp','fsi','fpi','fpp','fv']
       # else:
-      pres = ['fe','fsi','fpi','fpp','fv']
+      pres = ['fe','fsi','fpi','fpp']
+      if self.vdwnn:
+         pres.append('fv')
+
       for t in range(1,self.messages+1):
           pres.append('f'+str(t))
 
