@@ -114,7 +114,7 @@ def DIV_IF(y,x):
 class ReaxFF(object):
   def __init__(self,libfile='ffield',direcs={},
                dft='ase',atoms=None,
-               cons=['val','vale','lp3','cutoff','acut'], #,'hbtol'
+               cons=['val','vale','lp3','cutoff','hbtol'],# 'acut'
                opt=None,optword='nocoul',
                VariablesToOpt=None,
                nanv={'boc1':-2.0},
@@ -129,9 +129,10 @@ class ReaxFF(object):
                InitCheck=True,
                resetDeadNeuron=False,
                optmol=True,
-               mefac=0.1,
+               lambda_me=0.1,
                nn=False,
                nnopt=True,
+               vdwnn=False,
                bo_layer=[8,4],
                spec=[],
                sort=False,
@@ -166,7 +167,7 @@ class ReaxFF(object):
       self.cons          = cons
       self.optword       = optword
       self.optmol        = optmol
-      self.mefac         = mefac
+      self.lambda_me     = lambda_me
       self.vdwcut        = vdwcut
       self.dft           = dft
       self.atoms         = atoms
@@ -179,6 +180,7 @@ class ReaxFF(object):
       self.hblong        = hblong
       self.nn            = nn
       self.nnopt         = nnopt
+      self.vdwnn         = vdwnn
       self.bo_layer      = bo_layer
       self.weight        = weight
       self.spec          = spec
@@ -383,7 +385,7 @@ class ReaxFF(object):
       self.pbo,self.sbo,self.SBO,self.SBO12,self.SBO3,self.SBO01 = {},{},{},{},{},{}
       self.dang,self.D_ang = {},{}
 
-      self.thet,self.expang,self.f_7,self.f_8,self.rnlp = {},{},{},{},{}
+      self.thet,self.thet2,self.expang,self.f_7,self.f_8,self.rnlp = {},{},{},{},{},{}
       self.EPEN,self.BOij,self.BOjk,self.Epen,self.epen = {},{},{},{},{}
 
       self.expcoa1,self.texp0,self.texp1,self.texp2,self.texp3 = {},{},{},{},{}
@@ -517,10 +519,10 @@ class ReaxFF(object):
           self.ME += tf.square(self.MolEnergy[mols])
 
       self.loss_atol = self.supervise()
-
       self.Loss     += self.loss_atol
+      
       if self.optmol:
-         self.Loss  += self.ME*self.mefac
+         self.Loss  += self.ME*self.lambda_me
       self.accuracy  = self.accuracy/self.nmol
 
 
@@ -655,24 +657,35 @@ class ReaxFF(object):
              continue
           self.Dpi[sp]   = tf.gather_nd(self.DPIL,self.atomlist[sp])
 
-          self.get_eover(sp,self.delta[sp],self.Dpi[sp])
-          self.EOVER = self.EOV[sp] if i==0 else tf.concat((self.EOVER,self.EOV[sp]),0)
+          if not (self.optword.find('noover')>=0 and self.optword.find('nounder')>=0):
+             self.get_eover(sp,self.delta[sp],self.Dpi[sp])
+             self.EOVER = self.EOV[sp] if i==0 else tf.concat((self.EOVER,self.EOV[sp]),0)
+
+             self.get_eunder(sp,self.delta[sp],self.Dpi[sp])
+             self.EUNDER = self.EUN[sp] if i==0 else tf.concat((self.EUNDER,self.EUN[sp]),0)
 
-          self.get_eunder(sp,self.delta[sp],self.Dpi[sp])
-          self.EUNDER = self.EUN[sp] if i==0 else tf.concat((self.EUNDER,self.EUN[sp]),0)
           self.EATOM  = self.eatom[sp] if i==0 else tf.concat((self.EATOM,self.eatom[sp]),0)
           i += 1
 
       for mol in self.mols:
           mols = mol.split('-')[0] 
-          self.Elone[mol] = tf.gather_nd(self.ELONE,self.atomlink[mol])  
-          self.elone[mol] = tf.reduce_sum(input_tensor=self.Elone[mol],axis=0,name='lonepairenergy')
+          if self.optword.find('noover')>=0:
+          	 self.elone[mol] = 0.0
+          else:
+             self.Elone[mol] = tf.gather_nd(self.ELONE,self.atomlink[mol])  
+             self.elone[mol] = tf.reduce_sum(input_tensor=self.Elone[mol],axis=0,name='lonepairenergy')
 
-          self.Eover[mol] = tf.gather_nd(self.EOVER,self.atomlink[mol])  
-          self.eover[mol] = tf.reduce_sum(input_tensor=self.Eover[mol],axis=0,name='overenergy')
-
-          self.Eunder[mol] = tf.gather_nd(self.EUNDER,self.atomlink[mol])  
-          self.eunder[mol] = tf.reduce_sum(input_tensor=self.Eunder[mol],axis=0,name='underenergy')
+          if self.optword.find('noover')>=0:
+          	 self.eover[mol] = 0.0
+          else:
+             self.Eover[mol] = tf.gather_nd(self.EOVER,self.atomlink[mol])  
+             self.eover[mol] = tf.reduce_sum(input_tensor=self.Eover[mol],axis=0,name='overenergy')
+          
+          if self.optword.find('noover')>=0:
+          	 self.eunder[mol] = 0.0
+          else:
+             self.Eunder[mol] = tf.gather_nd(self.EUNDER,self.atomlink[mol])  
+             self.eunder[mol] = tf.reduce_sum(input_tensor=self.Eunder[mol],axis=0,name='underenergy')
     
           zpe_ = tf.gather_nd(self.EATOM,self.atomlink[mol]) 
           self.zpe[mol] = tf.reduce_sum(input_tensor=zpe_,name='zpe') + self.MolEnergy[mols]
@@ -868,7 +881,8 @@ class ReaxFF(object):
            self.get_theta0(ang)
 
       self.thet[ang]    = self.theta0[ang]-self.theta[ang]
-      self.expang[ang]  = tf.exp(-self.p['val2_'+ang]*tf.square(self.thet[ang]))
+      self.thet2[ang]   = tf.square(self.thet[ang])
+      self.expang[ang]  = tf.exp(-self.p['val2_'+ang]*self.thet2[ang])
       self.f_7[ang]     = self.f7(ang,atomj)
       self.f_8[ang]     = self.f8(ang,atomj)
       self.EANG[ang] = self.fijk[ang]*self.f_7[ang]*self.f_8[ang]*(self.p['val1_'+ang]-self.p['val1_'+ang]*self.expang[ang]) 
@@ -1169,12 +1183,13 @@ class ReaxFF(object):
   def set_neurons(self):
       self.unit = 4.3364432032e-2
       self.p_g  = ['boc1','boc2','coa2','ovun6','lp1','lp3',
-                   'ovun7','ovun8','val6','val9','val10','tor2',
+                   'ovun7','ovun8','val6','tor2',
                    'tor3','tor4','cot2','coa4','ovun4',               # 
-                   'ovun3','val8','coa3','pen2','pen3','pen4','vdw1',
+                   'ovun3','val8','val9','val10',
+                   'coa3','pen2','pen3','pen4','vdw1',
                    'cutoff','acut','hbtol'] # #
                    # 'trip2','trip1','trip4','trip3' ,'swa','swb'
-                   # tor3,tor4>0
+                   # tor3,tor4>0 
 
       self.p_spec = ['valboc','ovun5',
                      'lp2','boc4','boc3','boc5','rosi','ropi','ropp',
@@ -1222,6 +1237,17 @@ class ReaxFF(object):
 
       if self.optword.find('novdw')>=0:
          cons = cons + ['gammaw','vdw1','rvdw','Devdw','alfa'] 
+      if self.optword.find('noover')>=0:
+         cons = cons + ['ovun1'] # ,'ovun2','ovun3','ovun4'
+      if self.optword.find('nounder')>=0:
+         cons = cons + ['ovun5','ovun6','ovun7','ovun8'] 
+      if self.optword.find('noover')>=0 and self.optword.find('nounder')>=0:
+         cons = cons + ['ovun2','ovun3','ovun4'] 
+      if self.optword.find('nolone')>=0:
+         cons = cons + ['lp2','lp3'] # 'lp1'
+      if self.optword.find('nohb')>=0:
+         cons = cons + ['Dehb','rohb','hb1','hb2','hbtol'] 
+
       self.tor_v = ['tor2','tor3','tor4','V1','V2','V3','tor1','cot1','cot2'] 
 
       if self.optword.find('notor')>=0:
@@ -1257,11 +1283,12 @@ class ReaxFF(object):
           such as: val3,val6,val7,be1,lp1,ovun4,hb2,pen2
       '''   
       self.clip_pe = ['ovun3','ovun4','ovun6','ovun7',
-                      'val2','val3','val4',
-                      'val6','val8','val9','val10',
+                      'val4',
+                      'val6','val8','val10',
                       'pen2','pen3','pen4','coa4',
                       'theta0']  
-                      # 'Depi','Depp','ovun1','ovun2','ovun5',
+                      # 'Depi','Depp','ovun1','ovun2','ovun5','val2', 'val3',
+                      # 'val9'
       for k in self.v:
           vn  = k.split('_')
           key = vn[0]
@@ -1283,9 +1310,9 @@ class ReaxFF(object):
           elif key=='cutoff':
              self.p[k] = tf.clip_by_value(self.v[k],0.009999,0.012)
           elif key=='ovun5':
-             self.p[k] = tf.clip_by_value(self.v[k],0.0,60.0*self.unit)
+             self.p[k] = tf.clip_by_value(self.v[k],0.0,999.0*self.unit)
           elif key=='ovun1':
-             self.p[k] = tf.clip_by_value(self.v[k],0.0,2.0)
+             self.p[k] = tf.clip_by_value(self.v[k],0.0,10.0)
           elif key in ['boc4','boc5','vdw1']:
              self.p[k] = tf.clip_by_value(self.v[k],0.00000001,100.0)
           elif key == 'boc3':
@@ -1297,17 +1324,21 @@ class ReaxFF(object):
           elif key == 'pen1':
              self.p[k] = tf.clip_by_value(self.v[k],-69.0*self.unit,199.0*self.unit)
           elif key == 'Devdw':
-             self.p[k] = tf.clip_by_value(self.v[k],0.001*self.unit,0.333*self.unit)
+             D_= 600.0 if self.vdwnn else 0.8
+             self.p[k] = tf.clip_by_value(self.v[k],0.01*self.unit,D_*self.unit)
           elif key == 'Dehb':
              self.p[k] = tf.clip_by_value(self.v[k],-3.0*self.unit,3.0*self.unit)
           elif key == 'Desi':
-             self.p[k] = tf.clip_by_value(self.v[k],10.0*self.unit,1000.0*self.unit)
+             if self.nn:
+                self.p[k] = tf.clip_by_value(self.v[k],1.0*self.unit,999.0*self.unit)
+             else:
+                self.p[k] = tf.clip_by_value(self.v[k],100.0*self.unit,999.0*self.unit)
           elif key== 'Depi':
              self.p[k] = tf.clip_by_value(self.v[k],0.000*self.unit,299.0*self.unit)
           elif key== 'Depp':
              self.p[k] = tf.clip_by_value(self.v[k], 0.000*self.unit,299.0*self.unit)
           elif key=='tor1':
-             self.p[k] = tf.clip_by_value(self.v[k],-99.9900,-0.0)
+             self.p[k] = tf.clip_by_value(self.v[k],-39.9900,-0.0)
           elif key== 'ovun2':
              self.p[k] = tf.clip_by_value(self.v[k],-39.00,-0.01)
           elif key in ['tor3','val7']:
@@ -1324,21 +1355,23 @@ class ReaxFF(object):
           elif key in ['bo1','bo3','bo5']:
              if self.nn:
                 if key =='bo1':
-                   self.p[k] = tf.clip_by_value(self.v[k],-0.2,-0.15)
+                   self.p[k] = tf.clip_by_value(self.v[k],-0.35,-0.15)
                 elif key =='bo3':
-                   self.p[k] = tf.clip_by_value(self.v[k],-0.15,-0.1)
+                   self.p[k] = tf.clip_by_value(self.v[k],-0.35,-0.1)
                 elif key =='bo5':
-                   self.p[k] = tf.clip_by_value(self.v[k],-0.15,-0.1)
+                   self.p[k] = tf.clip_by_value(self.v[k],-0.35,-0.05)
+                # self.p[k] = tf.clip_by_value(self.v[k],-0.35,-0.25)
              else:
                 self.p[k] = tf.clip_by_value(self.v[k],-50.0,-0.0001)
           elif key in ['bo2','bo4','bo6']:
              if self.nn:
                 if key =='bo2':
-                   self.p[k] = tf.clip_by_value(self.v[k],6.0,7.0)
+                   self.p[k] = tf.clip_by_value(self.v[k],3.5,8.5)
                 elif key =='bo4':
-                   self.p[k] = tf.clip_by_value(self.v[k],5.0,6.0)
+                   self.p[k] = tf.clip_by_value(self.v[k],1.5,8.5)
                 elif key =='bo6':
-                   self.p[k] = tf.clip_by_value(self.v[k],4.0,5.0)
+                   self.p[k] = tf.clip_by_value(self.v[k],1.0,8.5)
+                # self.p[k] = tf.clip_by_value(self.v[k],4.5,5.0)
              else:
                 self.p[k] = tf.clip_by_value(self.v[k],0.0,50.0)
           elif key == 'boc1':
@@ -1350,9 +1383,9 @@ class ReaxFF(object):
           elif key== 'lp1':
              self.p[k] = tf.clip_by_value(self.v[k],10.0,50.0)
           elif key== 'lp2':
-             self.p[k] = tf.clip_by_value(self.v[k],0.0,39.0*self.unit)
+             self.p[k] = tf.clip_by_value(self.v[k],0.0,99.0*self.unit)
           elif key== 'lp3':
-             self.p[k] = tf.clip_by_value(self.v[k],10.0,75.0001)
+             self.p[k] = tf.clip_by_value(self.v[k],1.0,75.0001)
           elif key == 'be1':
              self.p[k] = tf.clip_by_value(self.v[k],-6.0,6.0)
           elif key == 'be2':
@@ -1375,9 +1408,15 @@ class ReaxFF(object):
              rvdw_ = 1.3*self.p_['rosi_'+vn[1]]
              self.p[k] = tf.clip_by_value(self.v[k],rvdw_,2.5*self.p_['rosi_'+vn[1]])
           elif key=='val5':
-             self.p[k] = tf.clip_by_value(self.v[k],0.0,3.0)
+             self.p[k] = tf.clip_by_value(self.v[k],0.0,10.0)
+          elif key=='val3':
+             self.p[k] = tf.clip_by_value(self.v[k],0.0,8.0)
           elif key=='val1':
-             self.p[k] = tf.clip_by_value(self.v[k],0.0,99.0)
+             self.p[k] = tf.clip_by_value(self.v[k],0.0,200.0*self.unit)
+          elif key=='val2':
+             self.p[k] = tf.clip_by_value(self.v[k],0.3,8.0)
+          elif key=='val9':
+             self.p[k] = tf.clip_by_value(self.v[k],0.0,8.0)
           elif key=='val':
              sp = vn[1]
              self.p[k] = tf.clip_by_value(self.v[k],0.2,8.0)
@@ -1406,6 +1445,18 @@ class ReaxFF(object):
           if self.optword.find('notor')>=0:
              if key in ktor:
                 self.p_[k] = 0.0
+          if self.optword.find('nolone')>=0:
+             if key in 'lp2':
+                self.p_[k] = 0.0
+          if self.optword.find('noover')>=0:
+             if key in 'ovun1':
+                self.p_[k] = 0.0
+          if self.optword.find('nounder')>=0:
+             if key in 'ovun5':
+                self.p_[k] = 0.0
+          if self.optword.find('noang')>=0:
+             if key in ['val1','coa1','pen1']:
+                self.p_[k] = 0.0
 
           if key == 'zpe':
              continue
@@ -1453,11 +1504,20 @@ class ReaxFF(object):
           if self.optword.find('noang')>=0:
              if key in ['val1','coa1','pen1']:
                 self.p_[k] = 0.0
-                
+          if self.optword.find('nolone')>=0:
+             if key == 'lp2':
+                self.p_[k] = 0.0
+          if self.optword.find('noover')>=0:
+             if key == 'ovun1':
+                self.p_[k] = 0.0
+          if self.optword.find('nounder')>=0:
+             if key == 'ovun5':
+                self.p_[k] = 0.0
+
           if key == 'zpe':
              continue
           if key != 'n.u.':
-             if key in self.cons:
+             if key in self.cons or k in self.cons:
                 if key in self.punit:
                    self.v[k] = tf.constant(np.float32(self.unit*self.p_[k]),name=k)
                 else:
@@ -1695,7 +1755,7 @@ class ReaxFF(object):
              if accu>accMax:
                 accMax = accu
           if self.optmol:
-             loss_ = loss - latol - ME_*self.mefac
+             loss_ = loss - latol - ME_*self.lambda_me
           else:
              loss_ = loss - latol
              
@@ -1863,6 +1923,9 @@ class ReaxFF(object):
               'mf_layer':None,
               'be_layer':None,
               'vdw_layer':None,
+              'rcut':self.rcut,
+              'rcutBond':self.rcuta,
+              'rEquilibrium':self.re,
               'MolEnergy':self.MolEnergy_}
          js.dump(j,fj,sort_keys=True,indent=2)
          fj.close()
